package model

import (
	"kpt-pasture/util"
	"strconv"
	"strings"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
)

type WorkOrderUser struct {
	Id                int64                          `json:"id"`
	WorkOrderMasterId int64                          `json:"WorkOrderMasterId"`
	WorkOrderSubId    int64                          `json:"workOrderSubId"`
	UserId            int64                          `json:"userId"`
	StartTime         int64                          `json:"startTime"`
	ExecTime          int64                          `json:"execTime"`
	EndTime           int64                          `json:"endTime"`
	Status            pasturePb.WorkOrderStatus_Kind `json:"status"`
	IsShow            pasturePb.IsShow_Kind          `json:"isShow"`
	Remarks           string                         `json:"remarks"`
	CreatedAt         int64                          `json:"createdAt"`
	UpdatedAt         int64                          `json:"updatedAt"`
}

func (w *WorkOrderUser) TableName() string {
	return "work_order_user"
}

func NewWorkOrderUser(master *WorkOrderMaster, sub *WorkOrderSub) []*WorkOrderUser {
	res := make([]*WorkOrderUser, 0)
	userList := strings.Split(master.ExecPersons, ",")
	if len(userList) <= 0 {
		return res
	}
	startTime, _ := util.ConvertParseLocalUnix(master.ExecTime)
	for _, userIdStr := range userList {
		userId, _ := strconv.ParseInt(userIdStr, 10, 64)
		res = append(res, &WorkOrderUser{
			WorkOrderMasterId: master.Id,
			WorkOrderSubId:    sub.Id,
			UserId:            userId,
			StartTime:         startTime,
			Status:            pasturePb.WorkOrderStatus_Distribute,
			IsShow:            pasturePb.IsShow_Ok,
			Remarks:           "",
		})
	}

	return res
}
